//set up variables
#include <Servo.h>

Servo myservo;

int timeturn = 500;

int servopin = 9;
int servoangle;

int AIN1 = 12;
int AIN2 = 13;
int PWMA = 11;

int BIN1 = 4;
int BIN2 = 3;
int PWMB = 5;

int lightvalue;
int ambient = 270;
int lightthreshold = 700;

int light_levels[3];
int max_light = 0;
int max_light_index = 0;

float inches = 0;

int trigPin = 7;
int echoPin = 6;

int distance;
float distances[3];
float max_distance = 0;
int max_distance_index = 0;

void setup() {
  //set up inputs and outputs
  myservo.attach(servopin);
  pinMode(AIN1, OUTPUT);
  pinMode(AIN2, OUTPUT);
  pinMode(PWMA, OUTPUT);
  pinMode(BIN1, OUTPUT);
  pinMode(BIN2, OUTPUT);
  pinMode(PWMB, OUTPUT);

  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  Serial.begin(9600);

}

void loop() {
  lightvalue = analogRead(A0);
  myservo.write(0);
  //if light level is not detected, there must be no patient in need of medication
  if (lightvalue < ambient) {
    Serial.println("no patient detected");
    Serial.print("light value: ");
    Serial.println(lightvalue);
    delay(2000);
  }
  //if light level above ambient is detected, there must be a patient in need of medication
  else if (lightvalue < lightthreshold) {
    Serial.println("Not close enough to patient, continue searching");
    //search for brightest light source and get initial distances
    //turn left and save light level and distance to first array element
    turnleft(timeturn);
    light_levels[0] = analogRead(A0);
    distances[0] = getDistance();
    //turn right and save light level and distance to last array element
    turnright(timeturn);
    turnright(timeturn);
    light_levels[2] = analogRead(A0);
    distances[2] = getDistance();
    //turn to face middle and save light level and distanceto middle array element
    turnleft(timeturn);
    light_levels[1] = analogRead(A0);
    distances[1] = getDistance();

    Serial.print("left light: ");
    Serial.println(light_levels[0]);
    Serial.print("right light: ");
    Serial.println(light_levels[2]);
    Serial.print("middle light: ");
    Serial.println(light_levels[1]);

    Serial.print("left distance: ");
    Serial.println(distances[0]);
    Serial.print("right distance: ");
    Serial.println(distances[2]);
    Serial.print("middle distance: ");
    Serial.println(distances[1]);

    max_light = 0;
    max_light_index = 0;
    max_distance = 0;
    max_distance_index = 0;

    //for each element in the array compare light levels to find when light level is greatest
    for (int i = 0; i < 3; i++) {
      if (light_levels[i] > max_light) {
        max_light = light_levels[i];
        max_light_index = i;
      }
      if (distances[i] > max_distance) {
        max_distance = distances[i];
        max_distance_index = i;
      }
    }
    Serial.print("Max light: ");
    Serial.println(max_light);
    Serial.print("Max light index: ");
    Serial.println(max_light_index);

    Serial.print("Max distance: ");
    Serial.println(max_distance);
    Serial.print("Max distance index: ");
    Serial.println(max_distance_index);

    //if obstacle detected very close, move backward
    if (max_distance < 25) {
      Serial.println("obstacle detected, move backward and try to turn right");
      movebackward(1500, 200);
      turnright(timeturn);
    }
    //if light level is greatest to the left turn left
    if (max_light_index == 0 && distances[0] > 20) {
      turnleft(timeturn);
      Serial.println("greatest light to left");
      moveforward(max_distance * 10, 200);
    } 
    //if light level is greatest to the left, but obstacle exists and middle is open, move around obstacle and return to desired path
    else if (max_light_index == 0 && distances[1] > distances[0]) {
      Serial.println("greatest light to left, but path blocked, continue straight to avoid obstacle");
      moveforward(2000, 200);
      turnleft(timeturn);
      turnleft(timeturn);
      moveforward(2000, 200);
      turnright(timeturn);
    }
    //if light level is greatest to the right turn right
    else if (max_light_index == 2 && distances[2] > 20) {
      turnright(timeturn);
      Serial.println("greatest light to right");
      moveforward(max_distance * 10, 200);
    } 
    //if light level is greatest to the right, but obstacle exists and middle is open, move around obstacle and return to desired path
    else if (max_light_index == 2 && distances[1] > distances[2]) {
      Serial.println("greatest light to right, but path blocked, continue straight to avoid obstacle");
      moveforward(2000, 200);
      turnright(timeturn);
      turnright(timeturn);
      moveforward(2000, 200);
      turnleft(timeturn);
    }
    //if light level is greatest in the middle don't turn
    else if (max_light_index == 1 && distances[1] > 20) {
      Serial.println("greatest light in middle");
      moveforward(max_distance * 10, 200);
    } 
    //if light level is greatest to the middle, but obstacle exists and left is open, move around obstacle and return to desired path
    else if (max_light_index == 1 && distances[0] > distances[1]) {
      Serial.println("greatest light in middle, but path blocked, turn left to avoid obstacle");
      turnleft(timeturn);
      moveforward(2000, 200);
      turnright(timeturn);
      turnright(timeturn);
      moveforward(2000, 200);
      turnleft(timeturn);
    } 
    //if light level is greatest to the middle, but obstacle exists and right is open, move around obstacle and return to desired path
    else if (max_light_index == 1 && distances[2] > distances[1]) {
      Serial.println("greatest light in middle, but path blocked, turn right to avoid obstacle");
      turnright(timeturn);
      moveforward(2000, 200);
      turnleft(timeturn);
      turnleft(timeturn);
      moveforward(2000, 200);
      turnright(timeturn);
    }
  } 
  else {
    //if light is very bright, patient must be very close
    //stop moving and open medication delivery system
    Serial.println("patient is close");
    myservo.write(110);
    Serial.print("light value: ");
    Serial.println(lightvalue);
    delay(2000);
  }
}

//set up functions

float getDistance() {
  float echoTime;            //variable to store the time it takes for a ping to bounce off an object
  float calculatedDistance;  //variable to store the distance calculated from the echo time
  //send out an ultrasonic pulse that's 10ms long
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  echoTime = pulseIn(echoPin, HIGH);  //use the pulsein command to see how long it takes for the
  //pulse to bounce back to the sensor
  calculatedDistance = echoTime / 148.0;  //calculate the distance of the object that reflected the pulse
  // (half the bounce time multiplied by the speed of sound)
  return calculatedDistance;  //send back the distance that was calculated
}

void moveforward(int time, int speed) {
  //right wheel spin forward
  digitalWrite(AIN1, HIGH);
  digitalWrite(AIN2, LOW);

  //left wheel spin forward
  digitalWrite(BIN1, HIGH);
  digitalWrite(BIN2, LOW);

  //both wheels spin at passed in speed
  analogWrite(PWMA, speed);       //between 0 and 255
  analogWrite(PWMB, speed + 20);  //between 0 and 255

  //spin for passed in time
  delay(time);

  //stop moving
  movestop();

  //wait one second to ensure everything is stable
  delay(1000);
}

void movebackward(int time, int speed) {
  //right wheel spin backward
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, HIGH);

  //left wheel spin backward
  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, HIGH);

  //both wheels spin at passed in speed
  analogWrite(PWMA, speed);       //between 0 and 255
  analogWrite(PWMB, speed + 20);  //between 0 and 255

  //spin for passed in time
  delay(time);

  //stop moving
  movestop();

  //wait one second to ensure everything is stable
  delay(1000);
}

void turnright(int time) {
  //right wheel spin backward
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, HIGH);

  //left wheel spin forward
  digitalWrite(BIN1, HIGH);
  digitalWrite(BIN2, LOW);

  //both wheels spin
  analogWrite(PWMA, 200);  //between 0 and 255
  analogWrite(PWMB, 220);  //between 0 and 255

  //spin for passed in time
  delay(time);

  //stop moving
  movestop();

  //wait one second to ensure everything is stable
  delay(1000);
}

void turnleft(int time) {
  //right wheel spin forward
  digitalWrite(AIN1, HIGH);
  digitalWrite(AIN2, LOW);

  //left wheel spin backward
  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, HIGH);

  //both wheels spin
  analogWrite(PWMA, 200);  //between 0 and 255
  analogWrite(PWMB, 220);  //between 0 and 255

  //spin for passed in time
  delay(time);

  //stop moving
  movestop();

  //wait one second to ensure everything is stable
  delay(1000);
}

void movestop() {
  //both wheels stop moving
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, LOW);

  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, LOW);

  //both wheels don't spin
  analogWrite(PWMA, 0);  //between 0 and 255
  analogWrite(PWMB, 0);  //between 0 and 255
}